@@ -18,11 +18,9 @@ func init() {
 var commandDefinition = &cobra.Command{
 	Use:   "deletefile remote:path",
 	Short: `Remove a single file from remote.`,
-	Long: `
-Remove a single file from remote.  Unlike ` + "`" + `delete` + "`" + ` it cannot be used to
-remove a directory and it doesn't obey include/exclude filters - if the specified file exists,
-it will always be removed.
-`,
+	Long: `Remove a single file from remote.  Unlike ` + "`" + `delete` + "`" + ` it cannot be used to
+remove a directory and it doesn't obey include/exclude filters - if the
+specified file exists, it will always be removed.`,
 	Annotations: map[string]string{
 		"versionIntroduced": "v1.42",
 		"groups":            "Important",
