@@ -1,12 +1,11 @@
-//go:build linux || freebsd
-// +build linux freebsd
+//go:build linux
 
 // Package mount implements a FUSE mounting system for rclone remotes.
 package mount
 
 import (
 	"fmt"
-	"runtime"
+	"time"
 
 	"bazil.org/fuse"
 	fusefs "bazil.org/fuse/fs"
@@ -51,7 +50,7 @@ func mountOptions(VFS *vfs.VFS, device string, opt *mountlib.Options) (options [
 		options = append(options, fuse.WritebackCache())
 	}
 	if opt.DaemonTimeout != 0 {
-		options = append(options, fuse.DaemonTimeout(fmt.Sprint(int(opt.DaemonTimeout.Seconds()))))
+		options = append(options, fuse.DaemonTimeout(fmt.Sprint(int(time.Duration(opt.DaemonTimeout).Seconds()))))
 	}
 	if len(opt.ExtraOptions) > 0 {
 		fs.Errorf(nil, "-o/--option not supported with this FUSE backend")
@@ -70,9 +69,6 @@ func mountOptions(VFS *vfs.VFS, device string, opt *mountlib.Options) (options [
 // report an error when fusermount is called.
 func mount(VFS *vfs.VFS, mountpoint string, opt *mountlib.Options) (<-chan error, func() error, error) {
 	f := VFS.Fs()
-	if runtime.GOOS == "darwin" {
-		fs.Logf(nil, "macOS users: please try \"rclone cmount\" as it will be the default in v1.54")
-	}
 	if err := mountlib.CheckOverlap(f, mountpoint); err != nil {
 		return nil, nil, err
 	}
@@ -82,7 +78,7 @@ func mount(VFS *vfs.VFS, mountpoint string, opt *mountlib.Options) (<-chan error
 	fs.Debugf(f, "Mounting on %q", mountpoint)
 
 	if opt.DebugFUSE {
-		fuse.Debug = func(msg interface{}) {
+		fuse.Debug = func(msg any) {
 			fs.Debugf("fuse", "%v", msg)
 		}
 	}
