@@ -10,6 +10,7 @@ import (
 
 	"github.com/rclone/rclone/backend/crypt"
 	"github.com/rclone/rclone/fs"
+	"github.com/rclone/rclone/fs/accounting"
 	"github.com/rclone/rclone/fs/hash"
 	"github.com/rclone/rclone/fs/walk"
 )
@@ -119,7 +120,7 @@ func newListJSON(ctx context.Context, fsrc fs.Fs, remote string, opt *ListJSONOp
 		lj.dirs = false
 	}
 	if opt.ShowEncrypted {
-		fsInfo, _, _, config, err := fs.ConfigFs(fsrc.Name() + ":" + fsrc.Root())
+		fsInfo, _, _, config, err := fs.ConfigFs(fs.ConfigStringFull(fsrc))
 		if err != nil {
 			return nil, fmt.Errorf("ListJSON failed to load config for crypt remote: %w", err)
 		}
@@ -194,6 +195,14 @@ func (lj *listJSON) entry(ctx context.Context, entry fs.DirEntry) (*ListJSONItem
 		}
 		item.Encrypted = path.Base(item.EncryptedPath)
 	}
+	if lj.opt.Metadata {
+		metadata, err := fs.GetMetadata(ctx, entry)
+		if err != nil {
+			fs.Errorf(entry, "Failed to read metadata: %v", err)
+		} else if metadata != nil {
+			item.Metadata = metadata
+		}
+	}
 	if do, ok := entry.(fs.IDer); ok {
 		item.ID = do.ID()
 	}
@@ -224,14 +233,6 @@ func (lj *listJSON) entry(ctx context.Context, entry fs.DirEntry) (*ListJSONItem
 				item.Tier = do.GetTier()
 			}
 		}
-		if lj.opt.Metadata {
-			metadata, err := fs.GetMetadata(ctx, x)
-			if err != nil {
-				fs.Errorf(x, "Failed to read metadata: %v", err)
-			} else if metadata != nil {
-				item.Metadata = metadata
-			}
-		}
 	default:
 		fs.Errorf(nil, "Unknown type %T in listing in ListJSON", entry)
 	}
@@ -283,7 +284,8 @@ func StatJSON(ctx context.Context, fsrc fs.Fs, remote string, opt *ListJSONOpt)
 			return nil, nil
 		}
 		// Check the root directory exists
-		_, err := fsrc.List(ctx, "")
+		entries, err := fsrc.List(ctx, "")
+		accounting.Stats(ctx).Listed(int64(len(entries)))
 		if err != nil {
 			return nil, err
 		}
@@ -291,7 +293,7 @@ func StatJSON(ctx context.Context, fsrc fs.Fs, remote string, opt *ListJSONOpt)
 	}
 
 	// Could be a file or a directory here
-	if lj.files {
+	if lj.files && !strings.HasSuffix(remote, "/") {
 		// NewObject can return the sentinel errors ErrorObjectNotFound or ErrorIsDir
 		// ErrorObjectNotFound can mean the source is a directory or not found
 		obj, err := fsrc.NewObject(ctx, remote)
@@ -314,11 +316,15 @@ func StatJSON(ctx context.Context, fsrc fs.Fs, remote string, opt *ListJSONOpt)
 		}
 	}
 	// Must be a directory here
+	//
+	// Remove trailing / as rclone listings won't have them
+	remote = strings.TrimRight(remote, "/")
 	parent := path.Dir(remote)
 	if parent == "." || parent == "/" {
 		parent = ""
 	}
 	entries, err := fsrc.List(ctx, parent)
+	accounting.Stats(ctx).Listed(int64(len(entries)))
 	if err == fs.ErrorDirNotFound {
 		return nil, nil
 	} else if err != nil {
