@@ -368,7 +368,7 @@ func Run(t *testing.T, opt *Opt) {
 		}
 		file1Contents string
 		file1MimeType = "text/csv"
-		file1Metadata = fs.Metadata{"rclone-test": "potato"}
+		file1Metadata = fs.Metadata{"rclonetest": "potato"}
 		file2         = fstest.Item{
 			ModTime: fstest.Time("2001-02-03T04:05:10.123123123Z"),
 			Path:    `hello? sausage/êé/Hello, 世界/ " ' @ < > & ? + ≠/z.txt`,
@@ -701,6 +701,7 @@ func Run(t *testing.T, opt *Opt) {
 					if opt.SkipLeadingDot && test.name == "leading dot" {
 						t.Skip("Skipping " + test.name)
 					}
+
 					// turn raw strings into Standard encoding
 					fileName := encoder.Standard.Encode(test.path)
 					dirName := fileName
@@ -1273,10 +1274,14 @@ func Run(t *testing.T, opt *Opt) {
 				assert.Equal(t, file2Copy.Path, dst.Remote())
 
 				// check that mutating dst does not mutate src
-				err = dst.SetModTime(ctx, fstest.Time("2004-03-03T04:05:06.499999999Z"))
-				if err != fs.ErrorCantSetModTimeWithoutDelete && err != fs.ErrorCantSetModTime {
-					assert.NoError(t, err)
-					assert.False(t, src.ModTime(ctx).Equal(dst.ModTime(ctx)), "mutating dst should not mutate src -- is it Copying by pointer?")
+				if !strings.Contains(fs.ConfigStringFull(f), "copy_is_hardlink") {
+					err = dst.SetModTime(ctx, fstest.Time("2004-03-03T04:05:06.499999999Z"))
+					if err != fs.ErrorCantSetModTimeWithoutDelete && err != fs.ErrorCantSetModTime {
+						assert.NoError(t, err)
+						// Re-read the source and check its modtime
+						src = fstest.NewObject(ctx, t, f, src.Remote())
+						assert.False(t, src.ModTime(ctx).Equal(dst.ModTime(ctx)), "mutating dst should not mutate src -- is it Copying by pointer?")
+					}
 				}
 
 				// Delete copy
@@ -1910,6 +1915,9 @@ func Run(t *testing.T, opt *Opt) {
 				}
 				t.Logf("Opening root remote %q path %q from %q", configName, configLeaf, subRemoteName)
 				rootRemote, err := fs.NewFs(context.Background(), configName)
+				if errors.Is(err, fs.ErrorCantListRoot) {
+					t.Skip("Can't list from root on this remote")
+				}
 				require.NoError(t, err)
 
 				file1Root := file1
